@@ -9,6 +9,7 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "qjsonarray.h"
 #ifdef INTERNAL_TERMINAL
 #include <qtermwidget5/qtermwidget.h>
 #endif
@@ -27,6 +28,8 @@
 
 #include "filedialog.h"
 
+#include <QJsonDocument>
+
 const QString ShortcutDelegate::addRowButton = "<internal: add row>";
 const QString ShortcutDelegate::deleteRowButton = "<internal: delete row>";
 
@@ -203,8 +206,8 @@ void ShortcutDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
 			return;
 		}
 		QString value_alternative = QKeySequence(box->currentText()).toString(QKeySequence::PortableText);
-		QRegExp rxCharKey("(Shift\\+)?."); // matches all single characters and single characters with shift like "Shift+A", should not match e.g. "F1" or "DEL"
-		if (rxCharKey.exactMatch(value_alternative)) {
+        QRegularExpression rxCharKey("^(Shift\\+)?.$"); // matches all single characters and single characters with shift like "Shift+A", should not match e.g. "F1" or "DEL"
+        if (value_alternative.indexOf(rxCharKey) == 0) {
 			if (!UtilsUi::txsConfirmWarning(ConfigDialog::tr("The shortcut you entered is a standard character key.\n"
 			                                        "You will not be able to type this character. Do you wish\n"
 			                                        "to set the key anyway?"))) {
@@ -233,8 +236,11 @@ void ShortcutDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
                 QTreeWidgetItem *item=li.at(k);
                 QString id = item->data(0, Qt::UserRole).toString();
                 QStringList ids=id.split("/");
-                if(ids.value(0,"")!=progType)
-                    li.removeAll(item);
+                if(ids.value(0,"")!=progType){
+                    if(progType=="pdf" || ids.value(0,"")=="pdf"){
+                        li.removeAll(item);
+                    }
+                }
             }
 			REQUIRE(treeWidget->topLevelItem(1));
 			REQUIRE(treeWidget->topLevelItem(1)->childCount() >= 1);
@@ -249,7 +255,10 @@ void ShortcutDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
 				if (UtilsUi::txsConfirmWarning(QString(ConfigDialog::tr("The shortcut <%1> is already assigned to the command:")).arg(value) + "\n" + duplicate + "\n\n" + ConfigDialog::tr("Do you wish to remove the old assignment and bind the shortcut to the new command?"))) {
 					//model->setData(mil[0],"",Qt::DisplayRole);
 					foreach (QTreeWidgetItem *twi, li) {
-						if (twi && twi->text(2) == value) twi->setText(2, "");
+						if (twi) {
+							if (twi->text(2) == value) twi->setText(2, "");
+                            if (twi->text(3) == value) twi->setText(3, "");
+						}
 					}
 				} else {
 					return;
@@ -434,7 +443,7 @@ ConfigDialog::ConfigDialog(QWidget *parent): QDialog(parent,Qt::Dialog|Qt::Windo
 	connect(UpdateChecker::instance(), SIGNAL(checkCompleted()), this, SLOT(refreshLastUpdateTime()));
 	refreshLastUpdateTime();
 
-	//pageditor
+	//pageEditor
 	populateComboBoxFont(false);
 	connect(ui.checkBoxShowOnlyMonospacedFonts, SIGNAL(toggled(bool)), this, SLOT(populateComboBoxFont(bool)));
 
@@ -480,7 +489,8 @@ ConfigDialog::ConfigDialog(QWidget *parent): QDialog(parent,Qt::Dialog|Qt::Windo
             .arg("<a href=\"https://extensions.openoffice.org/de/search?f[0]=field_project_tags%3A157\">OpenOffice</a>",
                  "<a href=\"https://extensions.libreoffice.org/?Tag%5B0%5D=50&q=&Tags%5B%5D=50\">LibreOffice</a>"));
 	ui.labelGetDic->setOpenExternalLinks(true);
-	//pagequick
+
+	//pageQuick
 	connect(ui.pushButtonGrammarWordlists, SIGNAL(clicked()), this, SLOT(browseGrammarWordListsDir()));
 	connect(ui.pushButtonGrammarLTPath, SIGNAL(clicked()), this, SLOT(browseGrammarLTPath()));
 	connect(ui.pushButtonGrammarLTJava, SIGNAL(clicked()), this, SLOT(browseGrammarLTJavaPath()));
@@ -548,7 +558,10 @@ ConfigDialog::ConfigDialog(QWidget *parent): QDialog(parent,Qt::Dialog|Qt::Windo
 #else
 		CONTENTS_DISABLED
 #endif
-	);
+    );
+    createIcon(tr("Collaborative Editing"), getRealIcon("config_editor"));
+    // tweak all comboboxes in adv. editor pane to not change on scroll wheel as it messes with scrolling through the pane (#2977)
+    tweakFocusSettings(ui.scrollAreaWidgetContents_2->children());
 
 	Q_ASSERT(ui.pagesWidget->count() == ui.contentsWidget->count());
 
@@ -578,16 +591,9 @@ ConfigDialog::ConfigDialog(QWidget *parent): QDialog(parent,Qt::Dialog|Qt::Windo
 	ui.treePossibleToolbarActions->setHeaderHidden(true);
 
 	ui.lineEditMetaFilter->setPlaceholderText(tr("(option filter)"));
+	ui.lineEditMetaFilter->setClearButtonEnabled(true);
 	connect(ui.lineEditMetaFilter, SIGNAL(textChanged(QString)), SLOT(metaFilterChanged(QString)));
 
-	// poppler preview
-#ifdef NO_POPPLER_PREVIEW
-	int l = ui.comboBoxDvi2PngMode->count();
-	ui.comboBoxDvi2PngMode->removeItem(l - 1);
-	l = ui.comboBoxPreviewMode->count();
-	ui.comboBoxPreviewMode->removeItem(l - 1);
-	// maybe add some possibility to disable some preview modes in poppler mode
-#endif
 	ui.labelScreenResolution->setText(labelSystemdpi);
 
 	// set-up GUI scaling
@@ -596,16 +602,38 @@ ConfigDialog::ConfigDialog(QWidget *parent): QDialog(parent,Qt::Dialog|Qt::Windo
 	connect(ui.tbRevertSymbol, SIGNAL(clicked()), this, SLOT(revertClicked()));
     connect(ui.tbRevertPDF, SIGNAL(clicked()), this, SLOT(revertClicked()));
 
-	// limit dialog size
-	QRect screen = QGuiApplication::primaryScreen()->geometry();
+    // ai chat
+    connect(ui.cbAIProvider, SIGNAL(currentIndexChanged(int)), this, SLOT(aiProviderChanged(int)));
+    connect(ui.pbRetrieveModels, &QPushButton::clicked, this, &ConfigDialog::retrieveModels);
+    connect(ui.pbResetAIURL, &QPushButton::clicked, this, &ConfigDialog::resetAIURL);
+    // fill in the known models
+    aiFillInKnownModels();
+    // enable/disable custom URL depending on aiProvider
+    enableCustomURLEditor(ui.cbAIProvider->currentIndex());
+
+    // collaborative editing
+    connect(ui.comboBoxCollaborativeTool, SIGNAL(currentIndexChanged(int)), this, SLOT(collaborativeEditingToolChanged(int))); // TODO: implement when needed
+    connect(ui.pbSelectCollaborativeToolPath, SIGNAL(clicked()), this, SLOT(browseCollaborativeToolPath()));
+    connect(ui.pbSelectClientFolder, &QPushButton::clicked, this, &ConfigDialog::browseCollaborativeClientFolder);
+
+}
+
+
+void ConfigDialog::showAndLimitSize() {
+	show();
+	QRect screen = QGuiApplication::primaryScreen()->availableGeometry();
 	if (!screen.isEmpty()) {
-		int nwidth = width(), nheight = height();
-		if (nwidth > screen.width()) nwidth = screen.width();
-		if (nheight > screen.height()) nheight = screen.height();
+		int nwidth = width(), nheight = height(),
+			heightDiff = frameGeometry().height() - geometry().height(),
+			maxHeight = screen.height() - heightDiff, maxWidth = screen.width();
+
+		if (nwidth > maxWidth) nwidth = maxWidth;
+		if (nheight > maxHeight) nheight = maxHeight;
 		if (nwidth == width() && nheight == height()) return;
+
 		resize(nwidth, nheight);
 		move(frameGeometry().right() > screen.right() ? screen.left() : x(),
-		     frameGeometry().bottom() > screen.bottom() ? screen.left() : y());
+		   frameGeometry().bottom() > screen.bottom() ? screen.top() : y());	
 	}
 }
 
@@ -657,7 +685,142 @@ void ConfigDialog::revertClicked()
         if (bt->objectName() == "tbRevertPDF") {
             ui.horizontalSliderPDF->setValue(16);
         }
-	}
+    }
+}
+/*!
+ * \brief adapt model list depending on ai provider
+ * \param[provider] 0: mistral
+ * 1: openai
+ * 2: custom provider
+ */
+void ConfigDialog::aiProviderChanged(int provider)
+{
+    ui.cbAIPreferredModel->setEditable(true);
+    QLineEdit *modelLineEdit = ui.cbAIPreferredModel->lineEdit();
+    switch(provider){
+    case 0:
+        ui.cbAIPreferredModel->clear();
+        ui.cbAIPreferredModel->addItem("open-mistral-7b");
+        ui.cbAIPreferredModel->addItem("open-mixtral-8x7b");
+        ui.cbAIPreferredModel->addItem("mistral-small-latest");
+        ui.cbAIPreferredModel->addItem("mistral-medium-latest");
+        ui.cbAIPreferredModel->addItem("mistral-large-latest");
+        ui.cbAIPreferredModel->setCurrentIndex(0);
+        modelLineEdit->setPlaceholderText("Enter model name (e.g., open-mistral-7b)");
+        break;
+    case 1:
+        ui.cbAIPreferredModel->clear();
+        ui.cbAIPreferredModel->addItem("gpt-4o-mini");
+        ui.cbAIPreferredModel->addItem("gpt-3.5-turbo");
+        ui.cbAIPreferredModel->addItem("gpt-4");
+        ui.cbAIPreferredModel->addItem("gpt-4o");
+        ui.cbAIPreferredModel->setCurrentIndex(0);
+        modelLineEdit->setPlaceholderText("Enter model name (e.g., gpt-4o)");
+        break;
+    default:
+        ui.cbAIPreferredModel->clear();
+        modelLineEdit->setPlaceholderText("Enter model name (e.g., llama-3.3-8B-Instruct)");
+        break;
+    }
+    enableCustomURLEditor(provider);
+}
+/*!
+ * \brief enable ui elements when provider is custom
+ * disable otherwise
+ * \param provider
+ */
+void ConfigDialog::enableCustomURLEditor(int provider)
+{
+    ui.leAIAPIURL->setEnabled(provider==2);
+    ui.pbResetAIURL->setEnabled(provider==2);
+}
+/*!
+ * \brief retieve the current list of available model from AI provider
+ * API key is needed
+ */
+void ConfigDialog::retrieveModels()
+{
+
+    QString provider=ui.cbAIProvider->currentText();
+    QString key=ui.leAIAPIKey->text();
+    QString url;
+
+    switch(ui.cbAIProvider->currentIndex()){
+    case 0:
+        url="https://api.mistral.ai/v1/models";
+        break;
+    case 1:
+        url="https://api.openai.com/v1/models";
+        break;
+    case 2:
+        url=ui.leAIAPIURL->text();
+        url=url.replace("chat/completions","models");
+        break;
+    default:
+        break;
+    }
+    QNetworkRequest request(url);
+    if(!key.isEmpty()){
+        request.setRawHeader("Authorization",QString("Bearer "+key).toUtf8());
+    }
+    QNetworkAccessManager *manager = new QNetworkAccessManager(this);
+    connect(manager,&QNetworkAccessManager::finished,this,&ConfigDialog::modelsRetrieved);
+    manager->get(request);
+}
+/*!
+ * \brief reset custom ai api url to default
+ */
+void ConfigDialog::resetAIURL()
+{
+    ui.leAIAPIURL->setText("http://localhost:8080/v1/chat/completions");
+}
+
+void ConfigDialog::modelsRetrieved(QNetworkReply *reply)
+{
+    if(reply->error() == QNetworkReply::NoError){
+        QByteArray data = reply->readAll();
+        QJsonDocument doc=QJsonDocument::fromJson(data);
+        QJsonObject obj=doc.object();
+        QStringList models;
+
+        QJsonArray lst=obj["data"].toArray();
+        foreach (const QJsonValue & value, lst) {
+            models.append(value.toObject()["id"].toString());
+        }
+
+        if(!models.isEmpty()){
+            QString currentText = ui.cbAIPreferredModel->currentText();
+            ui.cbAIPreferredModel->clear();
+            ui.cbAIPreferredModel->addItems(models);
+            ui.cbAIPreferredModel->setEditable(true);
+            
+            // Restore previous selection if it was custom
+            if(!currentText.isEmpty() && !models.contains(currentText)) {
+                ui.cbAIPreferredModel->setCurrentText(currentText);
+            }
+            
+            ConfigManager *config = dynamic_cast<ConfigManager *>(ConfigManagerInterface::getInstance());
+            if (config){
+                config->ai_knownModels=models;
+            }
+        }
+
+    }
+    reply->deleteLater();
+}
+/*!
+ * \brief fill in known models for AI provider
+ */
+void ConfigDialog::aiFillInKnownModels()
+{
+    ConfigManager *config = dynamic_cast<ConfigManager *>(ConfigManagerInterface::getInstance());
+    if (!config) return;
+    if(!config->ai_knownModels.isEmpty()){
+        ui.cbAIPreferredModel->clear();
+        ui.cbAIPreferredModel->addItems(config->ai_knownModels);
+        ui.cbAIPreferredModel->setEditable(true);
+        ui.cbAIPreferredModel->setCurrentText(config->ai_preferredModel);
+    }
 }
 
 //sets the items of a combobox to the filenames and sub-directory names in the directory which name
@@ -769,6 +932,24 @@ void ConfigDialog::browsePathCommands()
 	UtilsUi::browse(ui.lineEditPathCommands, tr("Search Path for Commands"), "/", QDir::rootPath(), true);
 }
 
+void ConfigDialog::browseCollaborativeToolPath()
+{
+    UtilsUi::browse(ui.lineEditCollaborativeToolPath, tr("Search Path for Command"), "Executable (*)", QDir::rootPath(), false);
+}
+/*!
+ * \brief browse Collaborative Client Folder
+ */
+void ConfigDialog::browseCollaborativeClientFolder()
+{
+    UtilsUi::browse(ui.lineEditCollaborativeClientFolder, tr("Search Folder for Collaborative Client"), "/", QDir::rootPath(), false);
+}
+
+void ConfigDialog::collaborativeEditingToolChanged(int index)
+{
+    // no used
+    qDebug()<<"not implemented";
+}
+
 void ConfigDialog::updateDefaultDictSelection(const QString &dictPaths, const QString &newDefault)
 {
 	QString lang = ui.comboBoxSpellcheckLang->currentText();
@@ -841,7 +1022,32 @@ void ConfigDialog::hideShowAdvancedOptions(QWidget *w, bool on)
 			}
 		}
 		hideShowAdvancedOptions(w, on);
-	}
+    }
+}
+/*!
+ * \brief change focus and eventfilter to avoid accidental scrollen
+ * See #2977
+ * \param objList
+ */
+void ConfigDialog::tweakFocusSettings(QObjectList objList)
+{
+    for(auto *elem:objList){
+        QComboBox *cb=qobject_cast<QComboBox *>(elem);
+        if(cb){
+            cb->setFocusPolicy(Qt::StrongFocus);
+            cb->installEventFilter(this);
+        }
+        QSpinBox *sb=qobject_cast<QSpinBox *>(elem);
+        if(sb){
+            sb->setFocusPolicy(Qt::StrongFocus);
+            sb->installEventFilter(this);
+        }
+        auto *gb=qobject_cast<QGroupBox *>(elem);
+        if(gb){
+            // look into sub-boxes
+            tweakFocusSettings(gb->children());
+        }
+    }
 }
 
 void ConfigDialog::advancedOptionsToggled(bool on)
@@ -868,7 +1074,7 @@ void ConfigDialog::advancedOptionsToggled(bool on)
 
 void ConfigDialog::advancedOptionsClicked(bool on)
 {
-	if (on) {
+	if (on & !riddled) {
 		if (!askRiddle()) ui.checkBoxShowAdvancedOptions->setChecked(false);
 		else riddled = true;
 	}
@@ -1140,7 +1346,7 @@ void ConfigDialog::importDictionary()
     QString filename = FileDialog::getOpenFileName(this, tr("Import Dictionary"), QString(), tr("OpenOffice Dictionary") + " (*.oxt)", nullptr, QFileDialog::DontResolveSymlinks);
 	if (filename.isNull()) return;
 
-	ConfigManager *config = dynamic_cast<ConfigManager *>(ConfigManagerInterface::getInstance());
+    ConfigManager *config = dynamic_cast<ConfigManager *>(ConfigManagerInterface::getInstance());
 	if (!config) return;
 
 	QString targetDir = config->parseDir("[txs-settings-dir]/dictionaries");
@@ -1194,7 +1400,27 @@ void ConfigDialog::populateComboBoxFont(bool onlyMonospaced)
 	}
 	// restore font setting if possible
 	int idx = ui.comboBoxFont->findText(currentFont);
-	if (idx >= 0) ui.comboBoxFont->setCurrentIndex(idx);
+    if (idx >= 0) ui.comboBoxFont->setCurrentIndex(idx);
+}
+/*!
+ * \brief eventFilter for combobbox
+ * This filters wheel eents on unfocused combobox to avoid unwanted change on scrolling
+ * See #2977
+ * \param obj
+ * \param event
+ * \return
+ */
+bool ConfigDialog::eventFilter(QObject *obj, QEvent *event)
+{
+    if ( event->type() == QEvent::Wheel) {
+        auto *wdg=qobject_cast<QWidget*>( obj );
+        if( wdg && !wdg->hasFocus() )
+        {
+            event->ignore();
+            return true;
+        }
+    }
+    return QObject::eventFilter( obj, event );
 }
 
 #ifdef INTERNAL_TERMINAL
