@@ -162,7 +162,8 @@ class QCE_EXPORT QDocument : public QObject
 			ShowNone		= 0x00,
 			ShowTrailing	= 0x01,
 			ShowLeading		= 0x02,
-			ShowTabs		= 0x04
+            ShowTabs		= 0x04,
+            ShowIndentGuides= 0x08
 		};
 
 		Q_DECLARE_FLAGS(WhiteSpaceMode, WhiteSpaceFlag)
@@ -174,7 +175,6 @@ class QCE_EXPORT QDocument : public QObject
 			DisableLineCache            = 0x04,
 			ForceQTextLayout            = 0x08,
             ForceSingleCharacterDrawing = 0x10,
-            QImageCache = 0x20
 		};
 
 		Q_DECLARE_FLAGS(WorkAroundMode, WorkAroundFlag)
@@ -188,9 +188,17 @@ class QCE_EXPORT QDocument : public QObject
 		Q_INVOKABLE void setText(const QString& s, bool allowUndo);
 
 		void load(const QString& file, QTextCodec* codec);
-		void startChunkLoading();
-		void stopChunkLoading();
-		void addChunk(const QString& txt);
+
+        enum SaveErrorCode
+        {
+            success,
+            fileNotWritable,
+            noBackupFilenameAvailable,
+            backupFileNotRestored,
+            writingFailed,
+        };
+
+        SaveErrorCode save(const QString& filename);
 
 		QString getFileName() const;
 		QFileInfo getFileInfo() const;
@@ -401,6 +409,7 @@ class QCE_EXPORT QDocument : public QObject
 
 		void formatsChange (int line, int lines);
 		void contentsChange(int line, int lines);
+        void changedText(int startLine,int startCol,int endLine,int endCol,const QString &text);
 
 		void widthChanged(int width);
 		void heightChanged(int height);
