@@ -331,12 +331,12 @@ void QDocumentCommand::updateCursorsOnDeletion(int line, int column, int prefixL
 //	qDebug("removing %i lines at (%i, %i) with (%i : %i) bounds", numLines, line, column, prefixLength, suffixLength);
 //erstes zeichen einer zeile: warning: removing 0 lines at (12, 0) with (1 : -1) bounds
 //zeile mit 10 textzeichen: warning: removing 1 lines at (13, 0) with (10 : 0) bounds
-	foreach ( QDocumentCursorHandle *ch, m_doc->impl()->m_autoUpdatedCursorList )
+    foreach ( QDocumentCursorHandle *ch, m_doc->impl()->m_autoUpdatedCursorList )
 	{
 		if ( ch == m_cursor || ch->document() != m_doc)
 			continue;
 
-	//	qDebug("[[watch:0x%x(%i, %i)]]", ch, ch->m_begLine, ch->m_begOffset);
+        // qDebug("[[watch:0x%x(%i, %i)]]", ch, ch->m_begLine, ch->m_begOffset);
 		int columnEnd = (numLines >= 1) ? suffixLength : column + prefixLength;
 		// TODO : better selection handling
 		if ( ch->hasSelection() )
@@ -432,19 +432,17 @@ void QDocumentCommand::removeLines(int after, int n)
 */
 void QDocumentCommand::updateTarget(int l, int offset)
 {
-	//QDocumentLineHandle *h = m_doc->impl()->at(l);
-	
 	// update command sender if any
 	if ( m_cursor )
 	{
-//		qDebug("moving cursor [0x%x:beg] from (%i, %i) to line (%i, %i) as updating",
-//					m_cursor,
-//					m_cursor->m_begLine,
-//					m_cursor->m_begOffset,
-//					l,
-//					offset
-//					);
-//
+        /*qDebug("moving cursor [0x%x:beg] from (%i, %i) to line (%i, %i) as updating",
+                    m_cursor,
+                    m_cursor->m_begLine,
+                    m_cursor->m_begOffset,
+                    l,
+                    offset
+                    );*/
+
 		while ( l && (offset < 0) )
 		{
 			--l;
@@ -523,10 +521,11 @@ void QDocumentCommand::markUndone(QDocumentLineHandle *h)
 	\param doc host document
 	\param p parent command
 */
-QDocumentInsertCommand::QDocumentInsertCommand(	int l, int offset,
-												const QString& text,
-												QDocument *doc,
-												QDocumentCommand *p)
+QDocumentInsertCommand::QDocumentInsertCommand(int l, int offset,
+                                               const QString& text,
+                                               QDocument *doc,
+                                               QDocumentCommand *p,
+                                               bool externalChange)
  : QDocumentCommand(Insert, doc, p)
 {
     QStringList lines;
@@ -545,6 +544,8 @@ QDocumentInsertCommand::QDocumentInsertCommand(	int l, int offset,
 	if ( !m_doc || text.isEmpty() )
 		qFatal("Invalid insert command");
 
+    m_data.externalChange=externalChange;
+
 	m_data.lineNumber = l;
 	m_data.startOffset = offset;
 
@@ -635,6 +636,18 @@ void QDocumentInsertCommand::redo()
 	m_doc->impl()->emitContentsChange(m_data.lineNumber, m_data.handles.count() + 1);
     m_doc->setProposedPosition(QDocumentCursor(m_doc,m_data.lineNumber+m_data.handles.size(),m_data.endOffset));
 
+    // emit text change for collaborative editing
+    if(!m_data.externalChange){ // avoid loops
+        QString text=m_data.begin;
+        if(m_data.handles.size()){
+            for(int i=0;i<m_data.handles.size()-1;i++)
+                text+='\n'+m_data.handles[i]->text();
+            text+='\n'+m_data.handles.last()->text().left(m_data.endOffset);
+        }
+        m_doc->impl()->emitContentsChange(m_data.lineNumber,m_data.startOffset,m_data.lineNumber,m_data.startOffset,text);
+    }
+
+
 	markRedone(hl, m_first);
 
 	foreach ( QDocumentLineHandle *h, m_data.handles )
@@ -665,14 +678,27 @@ void QDocumentInsertCommand::undo()
 	{
 		insertText(m_data.lineNumber, m_data.startOffset, m_data.end);
 	}
-
-	updateTarget(m_data.lineNumber, m_data.startOffset + m_undoOffset);
+    bool anchor=keepAnchor(); // workaround for #2084. text is inserted with anchor. Undo tries to recreate an anchor of not present text and fails completely (editor insertText)
+    setKeepAnchor(false);
+    updateTarget(m_data.lineNumber, m_data.startOffset + m_undoOffset);
+    setKeepAnchor(anchor);
 
 	updateCursorsOnDeletion(m_data.lineNumber, m_data.startOffset, m_data.begin.length(), m_data.handles.count(), m_data.endOffset);
 
 	m_doc->impl()->emitContentsChange(m_data.lineNumber, m_data.handles.count() + 1);
     m_doc->setProposedPosition(QDocumentCursor(m_doc,m_data.lineNumber,m_data.startOffset));
 
+    // emit text change for collaborative editing
+    if(!m_data.externalChange){ // avoid loops
+        if ( m_data.handles.count() ){
+            // multiline remove
+            m_doc->impl()->emitContentsChange(m_data.lineNumber,m_data.startOffset,m_data.lineNumber+m_data.handles.size(),m_data.endOffset,m_data.end);
+        }else{
+            // in line remove
+            m_doc->impl()->emitContentsChange(m_data.lineNumber,m_data.startOffset,m_data.lineNumber,m_data.startOffset+m_data.begin.size(),"");
+        }
+    }
+
 	markUndone(hl);
 
 	foreach ( QDocumentLineHandle *h, m_data.handles )
@@ -699,10 +725,11 @@ void QDocumentInsertCommand::undo()
 	\param doc host document
 	\param p parent command
 */
-QDocumentEraseCommand::QDocumentEraseCommand(	int bl, int bo,
-												int el, int eo,
-												QDocument *doc,
-												QDocumentCommand *p)
+QDocumentEraseCommand::QDocumentEraseCommand(int bl, int bo,
+                                             int el, int eo,
+                                             QDocument *doc,
+                                             QDocumentCommand *p,
+                                             bool externalChange)
  : QDocumentCommand(Erase, doc, p)
 {
 	if (el>m_doc->lines()-1) {
@@ -714,6 +741,8 @@ QDocumentEraseCommand::QDocumentEraseCommand(	int bl, int bo,
 
 	QDocumentConstIterator it = m_doc->impl()->begin() + bl; //index(start);
 
+    m_data.externalChange=externalChange;
+
 	m_data.lineNumber = bl;
 	m_data.startOffset = bo;
 
@@ -812,9 +841,9 @@ void QDocumentEraseCommand::redo()
 
 	if ( m_data.handles.isEmpty() )
 	{
-		removeText(m_data.lineNumber, m_data.startOffset, m_data.begin.count());
+        removeText(m_data.lineNumber, m_data.startOffset, m_data.begin.size());
 	} else {
-		removeText(m_data.lineNumber, m_data.startOffset, m_data.begin.count());
+        removeText(m_data.lineNumber, m_data.startOffset, m_data.begin.size());
 
         if ( m_data.endOffset != -1 ){
 			insertText(m_data.lineNumber, m_data.startOffset, m_data.end);
@@ -827,9 +856,20 @@ void QDocumentEraseCommand::redo()
 
 	updateTarget(m_data.lineNumber, m_data.startOffset + m_redoOffset);
 
-	updateCursorsOnDeletion(m_data.lineNumber, m_data.startOffset, m_data.begin.length(), m_data.handles.count(), m_data.endOffset);
+    updateCursorsOnDeletion(m_data.lineNumber, m_data.startOffset, m_data.begin.length(), m_data.handles.size(), m_data.endOffset);
 
-	m_doc->impl()->emitContentsChange(m_data.lineNumber, m_data.handles.count() + 1);
+    m_doc->impl()->emitContentsChange(m_data.lineNumber, m_data.handles.size() + 1);
+
+    // emit text change for collaborative editing
+    if(!m_data.externalChange){ // avoid loops
+        if ( m_data.handles.isEmpty() ){
+            // in line remove
+            m_doc->impl()->emitContentsChange(m_data.lineNumber,m_data.startOffset,m_data.lineNumber,m_data.startOffset+m_data.begin.size(),"");
+        }else{
+            // multi line remove
+            m_doc->impl()->emitContentsChange(m_data.lineNumber,m_data.startOffset,m_data.lineNumber+m_data.handles.size(),m_data.endOffset,"");
+        }
+    }
 
 	markRedone(hl, m_first);
 
@@ -868,11 +908,24 @@ void QDocumentEraseCommand::undo()
         }
 
 		m_doc->impl()->emitContentsChange(m_data.lineNumber, m_data.handles.count() + 1);
+        if(!m_data.externalChange){ // avoid loops
+            QString text=m_data.begin;
+            if(m_data.handles.size()){
+                for(int i=0;i<m_data.handles.size()-1;i++)
+                    text+='\n'+m_data.handles[i]->text();
+                text+='\n'+m_data.handles.last()->text().left(m_data.endOffset);
+            }
+            m_doc->impl()->emitContentsChange(m_data.lineNumber,m_data.startOffset,m_data.lineNumber,m_data.startOffset,text);
+        }
 	} else {
 
 		insertText(m_data.lineNumber, m_data.startOffset, m_data.begin);
 
 		m_doc->impl()->emitContentsChange(m_data.lineNumber, 1);
+        if(!m_data.externalChange){ // avoid loops
+            // in line remove
+            m_doc->impl()->emitContentsChange(m_data.lineNumber,m_data.startOffset,m_data.lineNumber,m_data.startOffset,m_data.begin);
+        }
 	}
 
 	if ( m_data.handles.count() )
