@@ -1,5 +1,4 @@
 #include "syntaxcheck.h"
-#include "latexdocument.h"
 #include "latexeditorview_config.h"
 #include "spellerutility.h"
 #include "tablemanipulation.h"
@@ -64,8 +63,23 @@ void SyntaxCheck::putLine(QDocumentLineHandle *dlh, StackEnvironment previous, T
 	mLinesEnqueuedCounter.ref();
 	mLinesLock.unlock();
 	//avoid reading of any results before this execution is stopped
-	//mResultLock.lock(); not possible under windows
-	mLinesAvailable.release();
+    //mResultLock.lock(); not possible under windows
+    mLinesAvailable.release();
+}
+/*!
+ * \brief remove all outstanding unckecked lines
+ * Clear queue as all lines are rechecked
+ */
+void SyntaxCheck::clearQueue()
+{
+    mLinesLock.lock();
+    mLines.clear();
+    int n=mLinesAvailable.available();
+    if(n>0){
+        mLinesAvailable.acquire(n-1);
+    }
+    mLinesLock.unlock();
+    // what to do with the semaphore ?
 }
 
 /*!
@@ -82,7 +96,7 @@ void SyntaxCheck::stop()
 */
 void SyntaxCheck::run()
 {
-	ltxCommands = new LatexParser();
+    ltxCommands = QSharedPointer<LatexParser>::create();
 
 	forever {
 		//wait for enqueued lines
@@ -93,17 +107,24 @@ void SyntaxCheck::run()
 			mLtxCommandLock.lock();
 			if (newLtxCommandsAvailable) {
 				newLtxCommandsAvailable = false;
-				*ltxCommands = newLtxCommands;
+                if(newLtxCommands){
+                    *ltxCommands = *newLtxCommands;
+                }
                 speller=newSpeller;
                 mReplacementList=newReplacementList;
                 mFormatList=newFormatList;
+                m_nonTextGrammarFormats=m_newNonTextGrammarFormats;
 			}
 			mLtxCommandLock.unlock();
 		}
 
 		// get Linedata
 		mLinesLock.lock();
-		SyntaxLine newLine = mLines.dequeue();
+        if(mLines.isEmpty()){
+            mLinesLock.unlock();
+            continue;
+        }
+        SyntaxLine newLine = mLines.dequeue();
 		mLinesLock.unlock();
 		// do syntax check
 		newLine.dlh->lockForRead();
@@ -131,14 +152,26 @@ void SyntaxCheck::run()
 		newLine.dlh->lockForWrite();
 		if (newLine.ticket == newLine.dlh->getCurrentTicket()) { // discard results if text has been changed meanwhile
             newLine.dlh->setCookie(QDocumentLine::LEXER_COOKIE,QVariant::fromValue<TokenList>(tl));
+            QList<QFormatRange>grammarOverlays=newLine.dlh->getOverlaysNoLock(m_nonTextGrammarFormats);
             foreach (const Error &elem, newRanges){
                 if(!mSyntaxChecking && (elem.type!=ERR_spelling) && (elem.type!=ERR_highlight) ){
                     // skip all syntax errors
                     continue;
                 }
-                int fmt= elem.type == ERR_spelling ? SpellerUtility::spellcheckErrorFormat : syntaxErrorFormat;
-                fmt= elem.type == ERR_highlight ? elem.format : fmt;
+                int fmt= (elem.type == ERR_spelling) ? SpellerUtility::spellcheckErrorFormat : syntaxErrorFormat;
+                fmt= (elem.type == ERR_highlight) ? elem.format : fmt;
                 newLine.dlh->addOverlayNoLock(QFormatRange(elem.range.first, elem.range.second, fmt));
+                // for ERR_highlight, remove grammarErrors
+                if(m_hideNonTextGrammarErrors && elem.type==ERR_highlight){
+                    for(int i = 0; i<grammarOverlays.size();++i){
+                        const QFormatRange &range=grammarOverlays.at(i);
+                        if(range.offset>=elem.range.first && range.offset<=elem.range.second){
+                            newLine.dlh->removeOverlayNoLock(range);
+                            grammarOverlays.removeAt(i);
+                            --i;
+                        }
+                    }
+                }
             }
             // add comment hightlight if present
             if(commentStart.first>=0){
@@ -153,6 +186,17 @@ void SyntaxCheck::run()
 			//if excessCols has changed the subsequent lines need to be rechecked.
             // don't on initial check
             if (cookieChanged) {
+                // handle runAway arguments
+                for (int i = 0; i < activeEnv.size(); i++) {
+                    if (activeEnv[i].runAway == 0) {
+                        activeEnv.remove(i);
+                        --i;
+                    }else{
+                        if(activeEnv[i].runAway>0){
+                            activeEnv[i].runAway = activeEnv[i].runAway - 1;
+                        }
+                    }
+                }
 				QVariant env;
 				env.setValue(activeEnv);
 				newLine.dlh->setCookie(QDocumentLine::STACK_ENVIRONMENT_COOKIE, env);
@@ -166,7 +210,6 @@ void SyntaxCheck::run()
 		newLine.dlh->deref(); //if deleted, delete now
 	}
 
-	delete ltxCommands;
 	ltxCommands = nullptr;
 }
 
@@ -241,12 +284,13 @@ QString SyntaxCheck::getErrorAt(QDocumentLineHandle *dlh, int pos, StackEnvironm
 * \brief set latex commands which are referenced for syntax checking
 * \param cmds
 */
-void SyntaxCheck::setLtxCommands(const LatexParser &cmds)
+void SyntaxCheck::setLtxCommands(QSharedPointer<LatexParser> cmds)
 {
 	if (stopped) return;
 	mLtxCommandLock.lock();
 	newLtxCommandsAvailable = true;
-	newLtxCommands = cmds;
+    newLtxCommands = QSharedPointer<LatexParser>::create();
+    *newLtxCommands = *cmds;
     mLtxCommandLock.unlock();
 }
 
@@ -271,6 +315,25 @@ void SyntaxCheck::enableSyntaxCheck(const bool enable){
     if (stopped) return;
     mSyntaxChecking=enable;
 }
+/*! \brief hide non text spelling errors
+ * \param hide
+ */
+void SyntaxCheck::setHideNonTextGrammarErrors(const bool hide)
+{
+    m_hideNonTextGrammarErrors=hide;
+}
+/*!
+ * \brief SyntaxCheck::setNonTextGrammarFormats
+ * \param formats
+ */
+void SyntaxCheck::setNonTextGrammarFormats(const QList<int> formats)
+{
+    if (stopped) return;
+    mLtxCommandLock.lock();
+    newLtxCommandsAvailable = true;
+    m_newNonTextGrammarFormats=formats;
+    mLtxCommandLock.unlock();
+}
 /*!
  * \brief set character/text replacementList for spell checking
  * \param replacementList Map for characater/text replacement prior to spellchecking words. E.g. "u -> ü when german is activated
@@ -362,16 +425,16 @@ int SyntaxCheck::topEnv(const QString &name, const StackEnvironment &envs, const
 * \param id if >=0 check if the env has the given id.
 * \return environment id of  found env otherwise 0
 */
-int SyntaxCheck::containsEnv(const LatexParser &parser, const QString &name, const StackEnvironment &envs, const int id)
+int SyntaxCheck::containsEnv(const QString &name, const StackEnvironment &envs, const int id)
 {
 	for (int i = envs.size() - 1; i > -1; --i) {
 		Environment env = envs.at(i);
 		if (env.name == name) {
 			if (id < 0 || env.id == id)
 				return env.id;
-		}
-		if (id < 0 && parser.environmentAliases.contains(env.name)) {
-			QStringList altEnvs = parser.environmentAliases.values(env.name);
+        }
+        if (id < 0 && ltxCommands->environmentAliases.contains(env.name)) {
+            QStringList altEnvs = ltxCommands->environmentAliases.values(env.name);
 			foreach (const QString &altEnv, altEnvs) {
 				if (altEnv == name)
 					return env.id;
@@ -389,7 +452,7 @@ int SyntaxCheck::containsEnv(const LatexParser &parser, const QString &name, con
  * \param envs stack of environements
  * \return math is active (true) or not (false)
  */
-bool SyntaxCheck::checkMathEnvActive(const LatexParser &parser, const StackEnvironment &envs)
+bool SyntaxCheck::checkMathEnvActive(const StackEnvironment &envs)
 {
     for (int i = envs.size() - 1; i > -1; --i) {
         Environment env = envs.at(i);
@@ -399,8 +462,8 @@ bool SyntaxCheck::checkMathEnvActive(const LatexParser &parser, const StackEnvir
         if (env.name == "text") {
                 return false;
         }
-        if (parser.environmentAliases.contains(env.name)) {
-            QStringList altEnvs = parser.environmentAliases.values(env.name);
+        if (ltxCommands->environmentAliases.contains(env.name)) {
+            QStringList altEnvs = ltxCommands->environmentAliases.values(env.name);
             foreach (const QString &altEnv, altEnvs) {
                 if (altEnv == "math")
                     return true;
@@ -418,8 +481,22 @@ bool SyntaxCheck::checkMathEnvActive(const LatexParser &parser, const StackEnvir
 */
 bool SyntaxCheck::checkCommand(const QString &cmd, const StackEnvironment &envs)
 {
-	for (int i = 0; i < envs.size(); ++i) {
+    bool textOrMathEnvUsed=false;
+    for (int i = envs.size()-1; i > -1; --i) {
 		Environment env = envs.at(i);
+        if(textOrMathEnvUsed){
+            if(env.name=="math" || env.name=="text" ) continue; // only the lowest text/math is valid as they can be used alternately
+            // look also for alias envs!
+            QStringList altEnvs = ltxCommands->environmentAliases.values(env.name);
+            bool skip=false;
+            foreach (const QString &altEnv, altEnvs) {
+                if (altEnv=="math" || altEnv=="text" ){
+                    skip=true;
+                    break;
+                }
+            }
+            if(skip) continue; // only the lowest text/math is valid as they can be used alternately
+        }
 		if (ltxCommands->possibleCommands.contains(env.name) && ltxCommands->possibleCommands.value(env.name).contains(cmd))
 			return true;
 		if (ltxCommands->environmentAliases.contains(env.name)) {
@@ -429,6 +506,9 @@ bool SyntaxCheck::checkCommand(const QString &cmd, const StackEnvironment &envs)
 					return true;
 			}
 		}
+        if(env.name=="math" || env.name=="text" ){
+            textOrMathEnvUsed=true; // only the lowest text/math is valid as they can be used alternately
+        }
 	}
 	return false;
 }
@@ -460,13 +540,15 @@ bool SyntaxCheck::equalEnvStack(StackEnvironment env1, StackEnvironment env2)
 */
 void SyntaxCheck::markUnclosedEnv(Environment env)
 {
+    if(!mSyntaxChecking) return; // skip when no syntax errors are to be shown
+
 	QDocumentLineHandle *dlh = env.dlh;
 	if (!dlh)
 		return;
 	dlh->lockForWrite();
 	if (dlh->getCurrentTicket() == env.ticket) {
 		QString line = dlh->text();
-		line = ltxCommands->cutComment(line);
+        line = cutComment(line);
 		QString cmd = "\\begin{" + env.name + "}";
 		int index = line.lastIndexOf(cmd);
 		if (index >= 0) {
@@ -532,7 +614,7 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
             */
         }
     }
-
+    QVector<QParenthesis> m_parens;
     // check command-words
 	for (int i = 0; i < tl.length(); i++) {
         Token &tk = tl[i];
@@ -540,6 +622,74 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
         // used for formula -> brace -> {....}
         if(!activeEnv.isEmpty() && activeEnv.top().endingColumn>=0 && tk.start>activeEnv.top().endingColumn){
             Environment env=activeEnv.pop();
+        }
+        // handle single command env stop e.g. \ExplSyntaxOff
+        if(tk.type==Token::command){
+            const QString word=tk.getText();
+            if(ltxCommands->possibleCommands["%endEnv"].contains(word)){
+                const QString envName=ltxCommands->environmentAliases.value(word);
+                if(activeEnv.top().name == envName){
+                    activeEnv.pop();
+                    continue;
+                }
+            }
+        }
+
+        if(!activeEnv.isEmpty() && activeEnv.top().name == "%expl3"){
+            // special treatment for expl3 commands in expl3 env
+            if((tk.type==Token::commandUnknown || tk.type==Token::command)&&tk.getText()!="\\\\"){ // special treatment for \\ , see #3877
+                // collect next parts
+                // e.g. \cs_new:Npn is split into \cs _ new : Npn
+                const int start = tk.start;
+                int end=tk.start+tk.length;
+                int colonPosition=-1;
+                for(++i;i<tl.length();++i){
+                    Token tk2= tl[i];
+                    if(end != tk2.start){
+                        // token does not adjoin previous one
+                        --i;
+                        break;
+                    }
+                    end+=tk2.length;
+                    if(tk2.type==Token::word){
+                        continue;
+                    }
+                    if(tk2.type==Token::punctuation){
+                        if(tk2.getText()=="_"){
+                            continue;
+                        }
+                        if(tk2.getText()==":"){
+                            colonPosition=end;
+                            continue;
+                        }
+                    }
+                    end-=tk2.length;
+                    --i;
+                    break; // unwanted element, stop joing for latex3 command
+                }
+                if(colonPosition>=0) {
+                    // highlight part after colon as math/number
+                    // highlight command part
+                    Error elem;
+                    elem.range = QPair<int, int>(start, colonPosition-start);
+                    elem.format=mFormatList["#pictureHighlight"];
+                    elem.type = ERR_highlight;
+                    newRanges.append(elem); // highlight
+                    // highlight after column
+                    elem.range = QPair<int, int>(colonPosition, end-colonPosition);
+                    elem.format=mFormatList["math"];
+                    elem.type = ERR_highlight;
+                    newRanges.append(elem); // highlight
+                }else{
+                    // ltx3 command w/o colon inside
+                    Error elem;
+                    elem.range = QPair<int, int>(start, end-start);
+                    elem.format=mFormatList["#pictureHighlight"];
+                    elem.type = ERR_highlight;
+                    newRanges.append(elem); // highlight
+                }
+            }
+            continue;
         }
 		// ignore commands in definition arguments e.g. \newcommand{cmd}{definition}
 		if (stackContainsDefinition(stack)) {
@@ -575,7 +725,7 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
 			QString word = line.mid(tk.start, tk.length);
 			QStringList forbiddenSymbols;
 			forbiddenSymbols<<"^"<<"_";
-            if(forbiddenSymbols.contains(word) && !checkMathEnvActive(*ltxCommands, activeEnv) && tk.subtype!=Token::formula){
+            if(forbiddenSymbols.contains(word) && !checkMathEnvActive(activeEnv) && tk.subtype!=Token::formula){
 				Error elem;
 				elem.range = QPair<int, int>(tk.start, tk.length);
 				elem.type = ERR_MathCommandOutsideMath;
@@ -593,7 +743,7 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
             }else{
                 elem.format=mFormatList["math"];
             }
-            if(tk.type==Token::braces){
+            if(tk.type==Token::braces || tk.type==Token::openBrace){
                 // add to active env
                 Environment env;
                 env.name = "math";
@@ -603,31 +753,49 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                 env.level = tk.level;
                 env.startingColumn=tk.start+1;
                 env.endingColumn=tk.start+tk.length-1;
+                if(tk.type==Token::openBrace){
+                    env.endingColumn=-1;
+                }
                 // avoid stacking same env (e.g. braces in braces, see #2411 )
                 Environment topEnv=activeEnv.top();
                 if(topEnv.name!=env.name)
                     activeEnv.push(env);
             }
+            if(tk.type==Token::closeBrace){
+                if(activeEnv.top().name=="math"){
+                    activeEnv.pop();
+                }
+            }
             newRanges.append(elem);
         }
         // force text != math when text command is used, i.e. \textbf in math env, see #2603
         if(tk.subtype==Token::text){
-            if(tk.type==Token::braces){
+            if(tk.type==Token::braces||tk.type==Token::openBrace){
                 // add to active env
                 // invalidates math env as active
                 Environment env;
                 env.name = "text";
-                env.id = 1; // to be changed
+                env.id = 1;
+                env.runAway = mRUNAWAYLIMIT;
                 env.dlh = dlh;
                 env.ticket = ticket;
                 env.level = tk.level;
                 env.startingColumn=tk.start+1;
-                env.endingColumn=tk.start+tk.length-1;
+                if(tk.type==Token::openBrace){
+                    env.endingColumn=-1;
+                }else{
+                    env.endingColumn=tk.start+tk.length-1;
+                }
                 // avoid stacking same env (e.g. braces in braces, see #2411 )
                 Environment topEnv=activeEnv.top();
                 if(topEnv.name!=env.name)
                     activeEnv.push(env);
             }
+            if(tk.type==Token::closeBrace){
+                if(activeEnv.top().name=="text"){
+                    activeEnv.pop();
+                }
+            }
         }
         // spell checking
         if (speller->inlineSpellChecking && tk.type == Token::word && (tk.subtype == Token::text || tk.subtype == Token::title || tk.subtype == Token::shorttitle || tk.subtype == Token::todo || tk.subtype == Token::none)  && speller) {
@@ -657,12 +825,12 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                 }
             }
             word = latexToPlainWordwithReplacementList(word, mReplacementList); //remove special chars
-            if (speller->hideNonTextSpellingErrors && (checkMathEnvActive(*ltxCommands, activeEnv)||containsEnv(*ltxCommands, "picture", activeEnv)) ){
+            if (speller->hideNonTextSpellingErrors && (checkMathEnvActive(activeEnv)||containsEnv("picture", activeEnv)||containsEnv("pictureHighlight", activeEnv)) ){
                 word.clear();
                 tk.ignoreSpelling=true;
             }else{
                 tk.ignoreSpelling=false;
-                if(containsEnv(*ltxCommands, "math", activeEnv)){
+                if(containsEnv("math", activeEnv)){
                     // in math env, highlight as math-text !
                     Error elem;
                     elem.type = ERR_highlight;
@@ -704,6 +872,8 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                 elem.format=mFormatList["&math"];
                 elem.range = QPair<int, int>(tk.start, tk.length);
                 newRanges.append(elem);
+                QParenthesis p(61,17,tk.start,tk.length);
+                m_parens.append(p);
                 continue;
 			}
 			if (ltxCommands->mathStopCommands.contains(word) && !activeEnv.isEmpty() && activeEnv.top().name == "math") {
@@ -726,6 +896,8 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                     elem.format=mFormatList["&math"];
                     elem.range = QPair<int, int>(tk.start, tk.length);
                     newRanges.append(elem);
+                    QParenthesis p(61,18,tk.start,tk.length);
+                    m_parens.append(p);
 				}// ignore mismatching mathstop commands
 				continue;
 			}
@@ -767,7 +939,7 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                     }
                 }
             }
-			if (ltxCommands->possibleCommands["user"].contains(word) || ltxCommands->customCommands.contains(word))
+            if (ltxCommands->possibleCommands["user"].contains(word))
 				continue;
 			if (!checkCommand(word, activeEnv)) {
 				Error elem;
@@ -874,19 +1046,12 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
 					if(env=="tikztimingtable"){
 						option="ll"; // is always 2 columns
 					}else{
-						for (int k = i + 1; k < tl.length(); k++) {
-							Token elem = tl.at(k);
-							if (elem.level < tk.level)
-								break;
-							if (elem.level > tk.level)
-								continue;
-							if (elem.subtype == Token::colDef) {
-								option = line.mid(elem.start + 1, elem.length - 2); // strip {}
-								break;
-							}
-						}
+                        option = Parsing::getArg(tl.mid(i+1),Token::colDef);
 					}
 				}
+                if(option.contains("colspec")){
+                    option=LatexTables::handleColSpec(option);
+                }
 				QSet<QString> translationMap=ltxCommands->possibleCommands.value("%columntypes");
 				QStringList res = LatexTables::splitColDef(option);
 				QStringList res2;
@@ -910,10 +1075,13 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
 
 
         if (tk.type == Token::command) {
-			QString word = line.mid(tk.start, tk.length);
-			if(!tk.optionalCommandName.isEmpty()){
+            QString word = line.mid(tk.start, tk.length);
+            if (word.contains('@')) {
+                continue; //ignore commands containg @
+            }
+            if(!tk.optionalCommandName.isEmpty() && !tk.optionalCommandName.contains("/")){
 				word=tk.optionalCommandName;
-			}
+            }
 			Token tkEnvName;
 
 			if (word == "\\begin" || word == "\\end") {
@@ -923,8 +1091,24 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
 					word = word + line.mid(tkEnvName.start, tkEnvName.length);
 				}
 			}
+            // special treatment for \ExplSyntaxOn, \ExplSyntaxOff
+            // \ProvidesExplPackage, \ProvidesExplClass and \ProvidesExplFile
+            // activate latex3 mode which ignores _ in commandnames
+            if(ltxCommands->possibleCommands["%beginEnv"].contains(word)){
+                const QString envName=ltxCommands->environmentAliases.value(word);
+                Environment env;
+                env.name = envName;
+                env.id = 1; // to be changed
+                env.dlh = dlh;
+                env.ticket = ticket;
+                env.level = tk.level;
+                env.startingColumn=tk.start+tk.length;
+                activeEnv.push(env);
+                continue;
+            }
+
             // special treatment for & in math
-            if(word=="&" && containsEnv(*ltxCommands, "math", activeEnv)){
+            if(word=="&" && containsEnv("math", activeEnv)){
                 Error elem;
                 elem.range = QPair<int, int>(tk.start, tk.length);
                 elem.type = ERR_highlight;
@@ -949,6 +1133,8 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                 elem.format=mFormatList["&math"];
                 elem.range = QPair<int, int>(tk.start, tk.length);
                 newRanges.append(elem);
+                QParenthesis p(61,17,tk.start,tk.length);
+                m_parens.append(p);
 				continue;
 			}
 			if (ltxCommands->mathStopCommands.contains(word) && !activeEnv.isEmpty() && activeEnv.top().name == "math") {
@@ -971,6 +1157,8 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                     elem.format=mFormatList["&math"];
                     elem.range = QPair<int, int>(tk.start, tk.length);
                     newRanges.append(elem);
+                    QParenthesis p(61,18,tk.start,tk.length);
+                    m_parens.append(p);
 				}// ignore mismatching mathstop commands
 				continue;
 			}
@@ -993,7 +1181,22 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                     }
 					continue;
 				}
-
+                // special treatment { \\ } in tblr (multirow cell)
+                if(word=="\\\\" && activeEnv.top().name=="tblr"){
+                    // check if this token lies with braces/none
+                    bool skipToken=false;
+                    for(int j=i-1;j>=0;--j){
+                        Token tk2=tl.at(j);
+                        if(tk2.type==Token::braces && tk2.subtype==Token::none && tk2.start+tk2.length>tk.start){
+                            // inside braces, ignore
+                            skipToken=true;
+                            break;
+                        }
+                    }
+                    if(skipToken){
+                        continue;
+                    }
+                }
 				if ((word == "\\\\") || (word == "\\tabularnewline")) {
 					if (activeEnv.top().excessCol < (activeEnv.top().id - 1)) {
 						Error elem;
@@ -1011,13 +1214,12 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
 					continue;
 				}
 				if (word == "\\multicolumn") {
-					QRegExp rxMultiColumn("\\\\multicolumn\\{(\\d+)\\}\\{.+\\}\\{.+\\}");
-					rxMultiColumn.setMinimal(true);
-					int res = rxMultiColumn.indexIn(line, tk.start);
-					if (res > -1) {
+                    static QRegularExpression rxMultiColumn("\\\\multicolumn\\{(\\d+?)\\}\\{.+?\\}\\{.+?\\}");
+                    QRegularExpressionMatch rxMultiColumnMatch = rxMultiColumn.match(line, tk.start);
+                    if (rxMultiColumnMatch.hasMatch()) {
 						// multicoulmn before &
 						bool ok;
-						int c = rxMultiColumn.cap(1).toInt(&ok);
+                        int c = rxMultiColumnMatch.captured(1).toInt(&ok);
 						if (ok) {
 							activeEnv.top().excessCol += c - 1;
 						}
@@ -1056,9 +1258,21 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                 }
             }
 
-			if (ltxCommands->possibleCommands["user"].contains(word) || ltxCommands->customCommands.contains(word))
+            if (ltxCommands->possibleCommands["user"].contains(word))
 				continue;
 
+            if(tk.subtype >= Token::specialArg){
+                // from multi element special argument
+                QString value = line.mid(tk.start, tk.length);
+                QString special = ltxCommands->mapSpecialArgs.value(int(tk.subtype - Token::specialArg));
+                if (!ltxCommands->possibleCommands[special].contains(value)) {
+                    Error elem;
+                    elem.range = QPair<int, int>(tk.start, tk.length);
+                    elem.type = ERR_unrecognizedKey;
+                    newRanges.append(elem);
+                }
+                continue;
+            }
 			if (!checkCommand(word, activeEnv)) {
 				Error elem;
 				if (tkEnvName.type == Token::braces) {
@@ -1098,7 +1312,7 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                 }
 			}
 		}
-		if (tk.type == Token::specialArg) {
+        if (tk.type >= Token::specialArg) {
 			QString value = line.mid(tk.start, tk.length);
 			QString special = ltxCommands->mapSpecialArgs.value(int(tk.type - Token::specialArg));
 			if (!ltxCommands->possibleCommands[special].contains(value)) {
@@ -1174,8 +1388,8 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
 		if (tk.subtype == Token::keyVal_val) {
 			//figure out keyval
 			QString word = line.mid(tk.start, tk.length);
-            if(word=="{"){
-                continue; // assume open brace is always valid
+            if(word=="{" || tk.type==Token::braces){
+                continue; // assume open brace is always valid or element in braces can't be checked (will get here again w/o braces)
             }
 			// first get command
             QString command = tk.optionalCommandName;
@@ -1235,18 +1449,45 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
 					if(options.startsWith("#")){
 						continue; // ignore type keys, like width#L
 					}
+                    if(options.endsWith("#c")){
+                        continue; // ignore values for syntax checking (#c)
+                    }
                     if(options.startsWith("%")){
                         if (!ltxCommands->possibleCommands[options].contains(word)) {
+                            // special treatment for %color (mix)
+                            if(options=="%color"){
+                                if(word=="!") continue;
+                                bool ok;
+                                word.toInt(&ok);
+                                if(ok) continue; // number !
+                            }
                             Error elem;
                             elem.range = QPair<int, int>(tk.start, tk.length);
                             elem.type = ERR_unrecognizedKeyValues;
                             newRanges.append(elem);
                         }
                     }else{
+                        if(options.contains(" ")){
+                            // special treatment for values with spaces, i.e. multi word values
+                            for (int k = i + 1; k < tl.length(); ++k) {
+                                Token tk_elem = tl.at(k);
+                                if(tk_elem.subtype!=Token::keyVal_val){
+                                    tk_elem=tl.at(k-1);
+                                    word=line.mid(tk.start,tk_elem.start+tk_elem.length-tk.start); // combine multiple keyVal_val tokens if present
+                                    i=k-1; // skip over those tokens
+                                    break;
+                                }
+                                if(k==tl.length()-1){
+                                    // last token
+                                    word=line.mid(tk.start,tk_elem.start+tk_elem.length-tk.start); // combine multiple keyVal_val tokens if present
+                                    i=k; // skip over those tokens
+                                }
+                            }
+                        }
                         QStringList l = options.split(",");
                         if (!l.contains(word)) {
                             Error elem;
-                            elem.range = QPair<int, int>(tk.start, tk.length);
+                            elem.range = QPair<int, int>(tk.start, word.length());
                             elem.type = ERR_unrecognizedKeyValues;
                             newRanges.append(elem);
                         }
@@ -1255,6 +1496,27 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
 			}
 		}
 	}
+    if(!m_parens.isEmpty()){
+        // merge original parenthesis vector with new additions
+        // skip duplicates
+        QVector<QParenthesis> original_parens=dlh->parenthesis();
+        QVector<QParenthesis> result;
+        int i=0;
+        for(int j=0;j<m_parens.length();++j){
+            while(i<original_parens.size() && original_parens[i].offset<m_parens[j].offset){
+                result<<original_parens[i];
+                ++i;
+            }
+            if(i<original_parens.size() && m_parens[j].offset==original_parens[i].offset){
+                ++i;
+            }
+            result<<m_parens[j];
+        }
+        for(;i<original_parens.size();++i){
+            result<<original_parens[i];
+        }
+        dlh->setParenthesis(result);
+    }
     if(!activeEnv.isEmpty()){
         //check active env for env highlighting (math,verbatim)
         QStack<Environment>::Iterator it=activeEnv.begin();
@@ -1265,7 +1527,11 @@ void SyntaxCheck::checkLine(const QString &line, Ranges &newRanges, StackEnviron
                 if(altEnvs.contains(key)){
                     Error elem;
                     int start= it->dlh==dlh ? it->startingColumn : 0;
-                    elem.range = QPair<int, int>(start, commentStart>=0 ? commentStart-start : line.length()-start);
+                    int length= it->endingColumn-start;
+                    if(length<0){
+                            length= commentStart>=0 ? commentStart-start : line.length()-start;
+                    }
+                    elem.range = QPair<int, int>(start, length);
                     elem.type = ERR_highlight;
                     elem.format=mFormatList.value(key);
                     newRanges.prepend(elem);  // draw this first and then other on top (e.g. keyword highlighting) !
