@@ -15,6 +15,7 @@
 #define Header_TexStudio
 
 #include "aichatassistant.h"
+#include "collaborationmanager.h"
 #include "mostQtHeaders.h"
 
 #include "bibtexparser.h"
@@ -197,7 +198,7 @@ private:
 	//toolbars
 	QAction *actSave, *actUndo, *actRedo;
 
-	QLabel *statusLabelMode, *statusLabelProcess, *statusLabelLanguageTool;
+    QLabel *statusLabelMode, *statusLabelProcess, *statusLabelLanguageTool, *statusLabelCollab;
 	QToolButton *statusTbLanguage;
 	QToolButton *statusTbEncoding;
 	QActionGroup *spellLanguageActions;
@@ -289,7 +290,7 @@ protected slots:
 	void fileNew(QString fileName = "");
 	void fileNewFromTemplate();
 	void fileMakeTemplate();
-	void fileOpen();
+    void fileOpen(QString currentDir="");
 	void fileRestoreSession(bool showProgress = true, bool warnMissing = true);
     void fileSave(const bool saveSilently = false,QEditor *editor=nullptr);
 	void fileSaveAll();
@@ -527,6 +528,21 @@ protected slots:
 	void analyseTextFormDestroyed();
 	void generateRandomText();
 
+    void startCollabServer();
+    void connectCollabServer();
+    void disconnectCollabServer();
+    void updateCollabCursors(QDocumentCursor cur, QString userId);
+    void removeCollabCursor(LatexDocument *doc, QString userId);
+    void removeAllCollabCursor(QString userId);
+    void updateCollabChanges(QDocumentCursor cur,QString changes,QString userName);
+    void updateCollaborationEditors(int startLine,int startCol,int endLine,int endCol,const QString& changes);
+    bool registerFileForCollab(const QString filename);
+    void collabClientFinished(int exitCode, QString m_errorMessage);
+    void guestServerSuccessfullyStarted();
+    void hostServerSuccessfullyStarted();
+    void updateCollabStatus();
+    void copyCollabLinkToClipboard();
+
 	bool loadLog();
 	void onCompileError();
 	void setLogMarksVisible(bool visible);
@@ -663,6 +679,7 @@ private slots:
 	void packageParserFinished();
 	void readinAllPackageNames();
     void packageListReadCompleted(std::set<QString> packages);
+
 protected:
 	void dragEnterEvent(QDragEnterEvent *event);
 	void dropEvent(QDropEvent *event);
@@ -726,6 +743,8 @@ protected:
 
     bool mThesaurusWasStarted=false;
 
+    CollaborationManager *collabManager = nullptr;
+
 public:
     Q_PROPERTY(QString clipboard READ clipboardText WRITE setClipboardText)
 	Q_INVOKABLE QString clipboardText(const QClipboard::Mode &mode = QClipboard::Clipboard) const;
